@@ -2,11 +2,10 @@
 
 use abscissa_core::{status_err, Command, Runnable, Shutdown};
 use anyhow::{bail, Result};
-
-use crate::{Application, RUSTIC_APP};
-
 use dialoguer::Password;
 
+use crate::{commands::get_repository, Application, RUSTIC_APP};
+
 use rustic_core::{ConfigOptions, KeyOptions, OpenStatus, Repository};
 
 /// `init` subcommand
@@ -33,9 +32,7 @@ impl Runnable for InitCmd {
 impl InitCmd {
     fn inner_run(&self) -> Result<()> {
         let config = RUSTIC_APP.config();
-
-        let po = config.global.progress_options;
-        let repo = Repository::new_with_progress(&config.repository, po)?;
+        let repo = get_repository(&config.repository)?;
 
         // Note: This is again checked in repo.init_with_password(), however we want to inform
         // users before they are prompted to enter a password
