@@ -43,6 +43,10 @@ pub(crate) struct LsCmd {
     #[clap(long, short = 'l')]
     long: bool,
 
+    /// show uid/gid instead of user/group
+    #[clap(long, long("numeric-uid-gid"))]
+    numeric_id: bool,
+
     /// Listing options
     #[clap(flatten)]
     ls_opts: LsOptions,
@@ -87,8 +91,7 @@ impl Summary {
 impl LsCmd {
     fn inner_run(&self) -> Result<()> {
         let config = RUSTIC_APP.config();
-
-        let repo = open_repository(&config)?.to_indexed()?;
+        let repo = open_repository(&config.repository)?.to_indexed()?;
 
         let node =
             repo.node_from_snapshot_path(&self.snap, |sn| config.snapshot_filter.matches(sn))?;
@@ -103,7 +106,7 @@ impl LsCmd {
             let (path, node) = item?;
             summary.update(&node);
             if self.long {
-                print_node(&node, &path);
+                print_node(&node, &path, self.numeric_id);
             } else {
                 println!("{path:?} ");
             }
@@ -126,7 +129,7 @@ impl LsCmd {
 ///
 /// * `node` - the node to print
 /// * `path` - the path of the node
-fn print_node(node: &Node, path: &Path) {
+fn print_node(node: &Node, path: &Path, numeric_uid_gid: bool) {
     println!(
         "{:>1}{:>9} {:>8} {:>8} {:>9} {:>12} {path:?} {}",
         match node.node_type {
@@ -142,8 +145,18 @@ fn print_node(node: &Node, path: &Path) {
             .mode
             .map(parse_permissions)
             .unwrap_or_else(|| "?????????".to_string()),
-        node.meta.user.clone().unwrap_or_else(|| "?".to_string()),
-        node.meta.group.clone().unwrap_or_else(|| "?".to_string()),
+        if numeric_uid_gid {
+            node.meta.uid.map(|uid| uid.to_string())
+        } else {
+            node.meta.user.clone()
+        }
+        .unwrap_or_else(|| "?".to_string()),
+        if numeric_uid_gid {
+            node.meta.gid.map(|uid| uid.to_string())
+        } else {
+            node.meta.group.clone()
+        }
+        .unwrap_or_else(|| "?".to_string()),
         node.meta.size,
         node.meta
             .mtime
