@@ -52,7 +52,7 @@ impl RestoreCmd {
     fn inner_run(&self) -> Result<()> {
         let config = RUSTIC_APP.config();
         let dry_run = config.global.dry_run;
-        let repo = open_repository(&config)?.to_indexed()?;
+        let repo = open_repository(&config.repository)?.to_indexed()?;
 
         let node =
             repo.node_from_snapshot_path(&self.snap, |sn| config.snapshot_filter.matches(sn))?;
